#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#

(define-module (ordo util keyword-args)
  #:use-module (ice-9 exceptions)
  #:export (keyword-arg
            select-keyword-args
            validate-keyword-args))

(define* (keyword-arg args kw #:optional (default #f))
  (cond
   ((< (length args) 2) default)
   ((equal? (car args) kw) (cadr args))
   (else (keyword-arg (cddr args) kw default))))

(define (select-keyword-args kwargs wanted)
  (let loop ((kwargs kwargs) (accum '()))
    (cond
     ((null? kwargs)
      (reverse accum))
     ((member (car kwargs) wanted)
      (loop (cddr kwargs) (cons* (car kwargs) (cadr kwargs) accum)))
     (else (loop (cddr kwargs) accum)))))

(define (validate-keyword-args kwargs)
  (unless (even? (length kwargs))
    (raise-exception
     (make-exception
      (make-programming-error)
      (make-exception-with-message "keyword args should have an even number of elements")))))
